<?php
/**
* Copyright (C) 2017-2019 Petr Hucik <petr@getdatakick.com>
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@getdatakick.com so we can send you a copy immediately.
*
* @author    Petr Hucik <petr@getdatakick.com>
* @copyright 2017-2019 Petr Hucik
* @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*/

namespace AdvDebug;

use Configuration;
use Tools;

class Settings
{
    const LICENSE = 'ADVDEBUG_LICENSE';
    const APP_URL = 'ADVDEBUG_APP_URL';
    const BACKEND_APP_URL = 'ADVDEBUG_BACK_APP_URL';
    const API_URL = 'ADVDEBUG_API_URL';
    const SETTINGS = 'ADVDEBUG_SETTINGS';
    const VERSION = 'ADVDEBUG_VERSION';
    const ACTIVATED = 'ADVDEBUG_ACTIVATED';
    const CHECK_VERSION = 'ADVDEBUG_CHECK_VERSION';

    const DAY = 86400;

    private $data;

    public function __construct()
    {
        $this->data = self::getDefaultSettings();
        $stored = Configuration::getGlobalValue(self::SETTINGS);
        if ($stored) {
            $stored = json_decode($stored, true);
            if ($stored) {
                $this->data = self::mergeSettings($this->data, $stored);
            }
        }
    }

    private static function getDefaultSettings()
    {
        return [
            'profiles' => [
            ],
        ];
    }

    public function init()
    {
        $settings = static::getDefaultSettings();
        $code1 = Tools::passwdGen(32);
        $code2 = Tools::passwdGen(32);
        $settings['profiles'] = [
            $code1 => [
                'name' => 'Permanent',
                'expiration' => static::DAY * 365 * 10
            ],
            $code2 => [
                'name' => 'One day',
                'expiration' => static::DAY
            ]
        ];
        return $this->set($settings);
    }

    public function reset()
    {
        return $this->remove() && $this->init();
    }


    public function isActivated()
    {
        return (bool)Configuration::getGlobalValue(self::ACTIVATED);
    }

    public function setActivated()
    {
        return Configuration::updateGlobalValue(self::ACTIVATED, 1);
    }

    public function getAppUrl($module)
    {
        $url = Configuration::getGlobalValue(self::APP_URL);
        if (!$url) {
            $version = self::getUnderscoredVersion($module);
            $url = $module->getPath("/views/js/advdebug-{$version}.js");
        }
        return $url;
    }

    public function getBackendAppUrl($module)
    {
        $url = Configuration::getGlobalValue(self::BACKEND_APP_URL);
        if (!$url) {
            $version = self::getUnderscoredVersion($module);
            $url = $module->getPath("views/js/back-{$version}.js");
        }
        return $url;
    }

    public function getApiUrl()
    {
        $url = Configuration::getGlobalValue(self::API_URL);
        return $url ? $url : 'https://api.getdatakick.com/';
    }

    public function getVersion()
    {
        return Configuration::getGlobalValue(self::VERSION);
    }

    public function setVersion($version)
    {
        Configuration::deleteByName(self::CHECK_VERSION);
        Configuration::updateGlobalValue(self::VERSION, $version);
    }

    public function getCheckModuleVersion()
    {
        $ret = Configuration::getGlobalValue(self::CHECK_VERSION);
        if ($ret) {
            return json_decode($ret, true);
        }
        return ['ts' => null, 'version' => null, 'notes' => null];
    }

    public function getProfile($code)
    {
        return $this->get(['profiles', $code]);
    }

    public function getProfiles()
    {
        return $this->get(['profiles']);
    }

    public function setCheckModuleVersion($version, $ts, $notes)
    {
        Configuration::updateGlobalValue(self::CHECK_VERSION, json_encode([
            'ts' => $ts,
            'version' => $version,
            'notes' => $notes,
        ]));
    }

    public function remove()
    {
        $this->data = null;
        Configuration::deleteByName(self::CHECK_VERSION);
        Configuration::deleteByName(self::SETTINGS);
        return true;
    }

    public function get($path = null)
    {
        $value = $this->data;
        if (is_null($path)) {
            return $value;
        }
        foreach ($path as $key) {
            if (isset($value[$key])) {
                $value = $value[$key];
            } else {
                die('ADVDEBUG: setting not found: ' . implode($path, '>'));
            }
        }
        return $value;
    }

    public function set($value)
    {
        $this->data = $value;
        return Configuration::updateGlobalValue(self::SETTINGS, json_encode($value));
    }

    private static function mergeSettings($left, $right)
    {
        $ret = [];
        foreach ($left as $key => $value) {
            if (isset($right[$key])) {
                if (is_array($value) && self::isAssoc($value)) {
                    $value = self::mergeSettings($value, $right[$key]);
                } else {
                    $value = $right[$key];
                }
            }
            $ret[$key] = $value;
        }
        return $ret;
    }

    private static function isAssoc(array $arr)
    {
        if (array() === $arr)
            return false;
        return array_keys($arr) !== range(0, count($arr) - 1);
    }

    private static function getUnderscoredVersion($module)
    {
        return str_replace('.', '_', $module->version);
    }
}
